/* $Id$
 * Copyright 2000-2002 SoundBite Corporation.   All Rights Reserved.
 */
package com.perforce.jdev;

import java.awt.Rectangle;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JPasswordField;
import javax.swing.JLabel;

import com.perforce.api.Env;

import oracle.ide.scm.SCMPropertyCustomizer;
import oracle.ide.scm.util.SCMAssert;
import oracle.ide.scm.util.SCMOperatingSystem;

/**
 * Describe class <code>P4PropertySheet</code> here.
 *
 * @author <a href="mailto:PKrause@PKRAUSE"></a>
 * @version 1.0
 */
public class P4PropertySheet extends JPanel implements SCMPropertyCustomizer {
   /** Defaults to create an environment */
   private final static Properties DEFAULT_PROPS = new Properties();

   /** Environment to hold properties */
   private final transient Env p4Env;

   /**
    * @serial default serialization of path to p4.exe
    */
   private JTextField execField = new JTextField();

   /**
    * @serial default serialization of p4 host and port number
    */
   private JTextField portField = new JTextField();

   /**
    * @serial default serialization of client name
    */
   private JTextField clientField = new JTextField();

   /**
    * @serial default serialization of user id
    */
   private JTextField userField = new JTextField();

   /**
    * @serial default serialization of user password
    */
   private JPasswordField passField = new JPasswordField();

   /**
    * @serial default serialization of "P4EXECUTABLE"
    */
   private JLabel execLabel = new JLabel();

   /**
    * @serial default serialization of "P4PORT"
    */
   private JLabel portLabel = new JLabel();

   /**
    * @serial default serialization of "P4CLIENT"
    */
   private JLabel clientLabel = new JLabel();

   /**
    * @serial default serialization of "P4USER"
    */
   private JLabel userLabel = new JLabel();

   /**
    * @serial default serialization of "P4PASSWORD"
    */
   private JLabel passLabel = new JLabel();


   /** Default Constructor (required for a JavaBean) */
   public P4PropertySheet()
   {
      this(defaultEnv());
   }

   /**
    * Constructs the P4PropertySheet for a p4 Env
    *
    * @param env Envrionment of current values
    */
   public P4PropertySheet(Env env)
   {
      p4Env = env;
      try {
         jbInit();
      } catch (Exception e) {
         final String msg = "Initialization failed";
         P4Client.logException(this, "<init>", msg, e);
         SCMAssert.fail(e);
      }
   }


   /**
    * Gets the value of the local host name property
    *
    * @return The LocalHostName value
    */
   private final static String getLocalHostName()
   {
      try {
         final String fullname = InetAddress.getLocalHost().getHostName();
         final int dot = fullname.indexOf('.');
         if (dot == -1) return fullname;
         return fullname.substring(0, dot);
      } catch (UnknownHostException e) {
         P4Client.logException(e, "getLocalHostName", "Ignored", e);
         return "mycomputer";
      }
   }

   /**
    * Description of the Method
    *
    * @return Description of the Returned Value
    */
   static Env defaultEnv()
   {
      return new Env(DEFAULT_PROPS);
   }


   /**
    * Generated method (implements the JDeveloper GUI editor pattern).
    *
    * @exception Exception if an error occurs
    */
   private void jbInit()
      throws Exception
   {
      this.setLayout(null);
      execLabel.setText("P4EXECUTABLE");
      execLabel.setBounds(new Rectangle(30, 50, 160, 15));
      portLabel.setText("P4PORT");
      portLabel.setBounds(new Rectangle(30, 95, 160, 15));
      clientLabel.setText("P4CLIENT");
      clientLabel.setBounds(new Rectangle(30, 140, 170, 15));
      userLabel.setText("P4USER");
      userLabel.setBounds(new Rectangle(30, 185, 170, 15));
      passLabel.setText("P4PASSWORD (optional)");
      passLabel.setBounds(new Rectangle(30, 230, 145, 15));
      execField.setText(p4Env.getExecutable());
      execField.setBounds(new Rectangle(200, 45, 175, 20));
      execField.setToolTipText("e.g. p4.exe");
      execField.addActionListener(
         new ActionListener() {
            public void actionPerformed(ActionEvent evt)
            {
               p4Env.setExecutable(execField.getText());
            }
         });
      portField.setText(p4Env.getPort());
      portField.setBounds(new Rectangle(200, 90, 175, 20));
      portField.setToolTipText("e.g. perforce:1666");
      portField.addActionListener(
         new ActionListener() {
            public void actionPerformed(ActionEvent evt)
            {
               p4Env.setPort(portField.getText());
            }
         });
      clientField.setBounds(new Rectangle(200, 135, 175, 20));
      clientField.setText(p4Env.getClient());
      clientField.setToolTipText("e.g. <user>-<host>-<view>");
      clientField.addActionListener(
         new ActionListener() {
            public void actionPerformed(ActionEvent evt)
            {
               p4Env.setClient(clientField.getText());
            }
         });
      userField.setBounds(new Rectangle(200, 180, 110, 20));
      userField.setText(p4Env.getUser());
      userField.setToolTipText(System.getProperty("user.name"));
      userField.addActionListener(
         new ActionListener() {
            public void actionPerformed(ActionEvent evt)
            {
               p4Env.setUser(userField.getText());
            }
         });
      passField.setText(p4Env.getPassword());
      passField.setBounds(new Rectangle(200, 225, 113, 20));
      passField.addActionListener(
         new ActionListener() {
            public void actionPerformed(ActionEvent evt)
            {
               final char[] pass = passField.getPassword();
               p4Env.setPassword(new String(pass));
            }
         });
      this.add(passLabel, null);
      this.add(userLabel, null);
      this.add(clientLabel, null);
      this.add(portLabel, null);
      this.add(execLabel, null);
      this.add(passField, null);
      this.add(userField, null);
      this.add(clientField, null);
      this.add(portField, null);
      this.add(execField, null);
   }

   /**
    * Gets the value of the env property
    *
    * @return The Env value
    */
   Env getEnv()
   {
      return p4Env;
   }

   /**
    * Sets the value of the properties property
    *
    * @param props The new properties value
    */
   public void setProperties(Map props)
   {
      final Set entries = props.entrySet();
      final Iterator entri = entries.iterator();
      while (entri.hasNext()) {
         final Map.Entry me = (Map.Entry) entri.next();
         p4Env.setProperty((String) me.getKey(), (String) me.getValue());
      }
   }

   public void validateProperties() {}

   /**
    * Gets the value of the component property
    *
    * @return The Component value
    */
   public Component getComponent()
   {
      return this;
   }

   /**
    * Gets the value of the properties property
    *
    * @return The Properties value
    */
   public Map getProperties()
   {
      return p4Env.getProperties();
   }

   // These defaults reflect the conventions in use at SoundBite.
   static {
      final String user = System.getProperty("user.name");
      final String host = getLocalHostName();
      final String view = "full";

      DEFAULT_PROPS.put("p4.user", user);
      DEFAULT_PROPS.put("p4.client",
         (user + "-" + host + "-" + view).toLowerCase());
      DEFAULT_PROPS.put("p4.port", "perforce:1666");
      DEFAULT_PROPS.put("p4.executable",
         SCMOperatingSystem.isUnix() ? "/usr/local/bin/p4" :
         "C:\\Program Files\\Perforce\\p4.exe");
   }

}
